#!/usr/bin/python3
""" A dummy mockup transport module for Cheap Pie """

# -*- coding: utf-8 -*-
## this file is part of cheap_pie, a python tool for chip validation
## author: Marco Merlin
## email: marcomerli@gmail.com

from ast import literal_eval

def hex_bw(val,hex_digits_width = 8 ):
    """ converts integer value into an hex string of fixed width """
    assert isinstance(val,int)

    retstr = "{0:#0{1}x}".format(val,hex_digits_width+2) # pylint: disable=C0209

    #  Explanation:
    # {   # Format identifier
    # 0:  # first parameter
    # #   # use "0x" prefix
    # 0   # fill with zeroes
    # {1} # to a length of n characters (including 0x), defined by the second parameter
    # x   # hexadecimal number, using lowercase letters for a-f
    # }   # End of format identifier

    # print(retstr)

    return retstr

def numeric_input(val):
    """ Numeric input that supports string evaluation with literal_eval """
    if isinstance(val, int):
        return val
    if isinstance(val, str):
        return int ( literal_eval(val) )
    assert False, f'Unsupported input type: {type(val)}'

def hifread_preproc(addr):
    """ convert input of hifread into numeric """
    return numeric_input(addr)

def hifwrite_preproc(addr,val,mask='0xFFFFFFFF'):
    """ convert input of hifwrite into numeric """
    return numeric_input(addr),numeric_input(val),numeric_input(mask)

class CpDummyTransport():
    """ A transport mockup """
    mem = {}

    def hifread(self, addr = "0x40000888"):
        """ Mockup for read a register """

        # make sure string format is always the same
        addr = hifread_preproc(addr)

        addrstr = hex_bw(addr)

        # check address exists, or create it
        if addrstr not in self.mem.keys(): # pylint: disable=C0201
            self.mem[addrstr] = 0

        return self.mem[addrstr]

    def hifwrite(self,addr = "0x40000888",val = "0x00000352"):
        """ Mockup for write a register """

        addr,val,_ = hifwrite_preproc(addr,val)

        self.mem[hex_bw(addr)] = val

        return int( val )

def test_cp_dummy( transport = CpDummyTransport() ):
    """ Test transport mockup """
    print("# Test dummy transport")
    addr = 4
    val = 5
    transport.hifwrite(addr=addr,val=val)
    assert val==transport.hifread(addr = addr)

if __name__ == '__main__':
    test_cp_dummy()



#!/usr/bin/python3

""" Cheap Pie module to convert .rdl register description into IP-XACT, UVM or verilog """

import sys
import os
import argparse
import pathlib

from systemrdl import RDLCompiler, RDLCompileError
from peakrdl.verilog import VerilogExporter

try:
    # newer version
    from peakrdl_ipxact import IPXACTExporter
    from peakrdl_uvm import UVMExporter
except ModuleNotFoundError: # not sure what should be put here
    # older version
    from peakrdl.ipxact import IPXACTExporter # pylint: disable=C0412
    from peakrdl.uvm import UVMExporter

def cli(args):
    """ Command Line parse to convert .rdl register description into IP-XACT, UVM or verilog """
    parser = argparse.ArgumentParser(description='rdl2any')

    # register format options
    parser.add_argument("-f", "--fname", help="register file description .rdl",
                        action='store', type = str,
                        default=os.path.join(
                                    pathlib.Path(__file__).parent.parent.absolute(),
                                    "devices",
                                    "rdl",
                                    "basic.rdl"
                                    )
                        )
    parser.add_argument("-ofmt", "--out-format", help="output format",
                        action='store', type = str, default="ipxact",
                        choices=["ipxact","uvm","vlog"])

    return parser.parse_args(args)

def rdl2any(args):
    """  convert .rdl register description into IP-XACT, UVM or verilog """
    prms = cli(args)
    rdlc = RDLCompiler()

    try:
        rdlc.compile_file(prms.fname)
        root = rdlc.elaborate()
    except RDLCompileError:
        sys.exit(1)

    if prms.out_format == 'ipxact':
        exporter = IPXACTExporter()
        ext = '.xml'
    elif prms.out_format == 'uvm':
        exporter = UVMExporter()
        ext = '.uvm'
    elif prms.out_format == 'vlog':
        exporter = VerilogExporter()
        ext = '.v'
    else:
        assert False, f'Unsupported output format {prms.out_format} !'

    base = os.path.splitext(prms.fname)[0]
    outfname = base + ext
    print('output file: ' + outfname)

    if prms.out_format == 'vlog':
        exporter.export(root, outfname, signal_overrides = {} )
    else:
        exporter.export(root, outfname )

    return outfname

def test_rdl2any():
    """ test convert .rdl register description into IP-XACT, UVM or verilog """
    print("# Test rdl2ipxact")
    rdl2any(['-ofmt','ipxact'])

    print("# Test rdl2uvm")
    rdl2any(['-ofmt','uvm'])

    print("# Test rdl2verilog")
    rdl2any(['-ofmt','vlog'])

if __name__ == '__main__':
    rdl2any(sys.argv[1:])



""" Core cheap pie classes """
from .cp_hal import CpHal
from .cp_builder import CpHalBuilder
from .cp_cli import cp_cli, cp_devices_fname, cp_devices_dir
from .cheap_pie_main import cp_main
from .test import *



#!/usr/bin/python3
"""
Cheap Pie Command Line Interface
"""
## this file is part of cheap_pie, a python tool for chip validation
## author: Marco Merlin
## email: marcomerli@gmail.com

import os
import sys
import pathlib
import argparse

def cp_root_dir():
    """ Returns the full path of Cheap Pie root directory """
    return pathlib.Path(__file__).parent.parent.absolute()

def cp_devices_dir():
    """ Returns the full path of ad filename in Cheap Pie devices directory """
    return os.path.join(
        cp_root_dir(),
        'devices')

def cp_devices_fname(fname):
    """ Returns the full path of ad filename in Cheap Pie devices directory """
    return os.path.join(
            cp_devices_dir(), fname
            )

def cp_cli(args=[]): # pylint: disable=W0102
    """
    Cheap Pie Command Line Interface
    """
    parser = argparse.ArgumentParser(description='Cheap Pie Configuration')
    # register format options
    parser.add_argument("-rf", "--regfname", help="register description file name",
                        action='store', type = str, default="QN908XC.svd")
    parser.add_argument("-dd", "--devicedir", help="register description files folder",
                        action='store', type = str,
                        default=cp_devices_dir()
                        )
    parser.add_argument("-fmt","--format", help="device description format",
                        action='store', type = str, default="cmsis-svd",
                        choices=["svd","cmsis-svd","ipxact","ipyxact","rdl"])
    parser.add_argument("-ve", "--vendor",
                        help="device vendor. if specified parses svd file from github repository.",
                        action='store', type = str, default=None)
    # transport options
    parser.add_argument("-d", "--device", help="jlink/ocd device name", action='store',
                        type = str, default="CORTEX-M4")
    parser.add_argument("-t" , "--transport", help="transport", action='store', type = str,
                        default="dummy", choices=["jlink","dummy","ocd","esptool","verilator"])
    parser.add_argument("-p", "--port", help="esptool serial port", action='store', type = str,
                        default="/dev/ttyUSB0")
    parser.add_argument("-topv", "--top_verilog",
                        help="top verilog file (when simulating with verilator)",
                        action='store', type = str, default="./devices/rdl/basic/basic_rf.sv")

    return parser.parse_args(args)

def test_cli(argv):
    """
    Test function for Cheap Pie Command Line Interface
    """
    return cp_cli(argv)

if __name__ == '__main__':
    print(test_cli(sys.argv[1:]))



#!/usr/bin/python3
""" Cheap Pie parser module for .svd files using SVDParser module """
## this file is part of cheap_pie, a python tool for chip validation
## author: Marco Merlin
## email: marcomerli@gmail.com

# parser build for CMSIS-SVD xml file format
# https://www.keil.com/pack/doc/CMSIS/SVD/html/index.html
#
# simplified manual initialization
#
# import untangle
# hal = untangle.parse('QN908XC.xml')
# from cbitfield import cp_bitfield
# from cp_register import cp_register
# ADC_ANA_CTRL = cp_register("ADC_ANA_CTRL","0x4000702C",
#                            "ADC core and reference setting regsiter" , hif )
# ADC_BM = cp_bitfield("ADC_BM","0x4000702C",
#                      "ADC_ANA_CTRL",3,0, "ADC bias current selection." ,hif)

from ast import literal_eval
from cmsis_svd.parser import SVDParser

from cheap_pie.cheap_pie_core.cp_builder import CpHalBuilder # pylint: disable=C0413,E0401
from cheap_pie.cheap_pie_core.cp_cli import cp_devices_fname # pylint: disable=C0413,E0401

def svd_parse_repo(fname,vendor=None,hif=None, base_address_offset = "0x00000000"):
    """ Cheap Pie parser function for .svd files using SVDParser module """
    ## read input file ########################################################
    if vendor is None:
        svd = SVDParser.for_xml_file(fname)
    else:
        svd = SVDParser.for_packaged_svd(vendor,fname)

    ## loop over lines ########################################################
    cpb = CpHalBuilder(hif)

    for periph in svd.get_device().peripherals:  # pylint: disable=R1702
        # print(periph.name.cdata)

        if hasattr(periph,'registers'):
            for reg in periph.registers:
                if hasattr( reg, 'name'):
                    # new register
                    regaddr=(
                        reg.address_offset +
                        periph.base_address +
                        literal_eval(base_address_offset)
                    )

                    cpb.reg_open(
                        regname = f'{periph.name}_{reg.name}',
                        regaddr = regaddr,
                        comments = reg.description
                        )

                    if hasattr(reg,'fields'):
                        for field in reg.fields:
                            if not field is None:
                                # Create new field class
                                cpb.newfield(
                                    regfield = field.name,
                                    width = field.bit_width,
                                    offset = field.bit_offset,
                                    comments = field.description,
                                )

    # convert output dictionary into structure
    return cpb.out()

def test_svd_parse_repo():
    """ Test Function for .svd parser based of SVDParser module """
    print('Testing QN9080 with repo parser...')

    fname = cp_devices_fname("QN908XC.svd")
    hal = svd_parse_repo(fname=fname)
    assert len(hal) > 0

    print('Testing K20 with repo parser...')
    hal = svd_parse_repo(fname='MK20D7.svd',vendor='Freescale')
    assert len(hal) > 0

if __name__ == '__main__':
    test_svd_parse_repo()



#!/usr/bin/python3
""" Cheap Pie module to parse .rdl files """
## this file is part of cheap_pie, a python tool for chip validation
## author: Marco Merlin
## email: marcomerli@gmail.com

from cheap_pie.parsers.ipyxact_parse import ipyxact_parse    # pylint: disable=C0413,E0401
from cheap_pie.tools.rdl2any import rdl2any                  # pylint: disable=C0413,E0401
from cheap_pie.cheap_pie_core.cp_cli import cp_devices_fname # pylint: disable=C0413,E0401

def rdl_parse(fname,hif=None,base_address_offset = "0x00000000" ):
    """ Cheap Pie function to parse .rdl files """

    # convert rdl to ipxact
    ipxact_fname = rdl2any(['-f',fname,'-ofmt','ipxact'])
    # parse ipxact
    return ipyxact_parse(ipxact_fname,hif=hif, base_address_offset=base_address_offset)

def test_rdl_parse():
    """ Cheap Pie function to test .rdl file parser """
    rdl_parse(cp_devices_fname("rdl/basic.rdl"))
    rdl_parse(cp_devices_fname("rdl/counter.rdl"))

if __name__ == '__main__':
    import sys
    if len(sys.argv) > 1:
        FNAME=sys.argv[1]
    else:
        FNAME=cp_devices_fname("rdl/basic.rdl")
    print(FNAME)
    hal = rdl_parse(FNAME)
    print(hal)



#!/usr/bin/python3
""" Cheap Pie native module parser for IP-XACT """
## this file is part of cheap_pie, a python tool for chip validation
## author: Marco Merlin
## email: marcomerli@gmail.com

from ast import literal_eval
import untangle # for parsing xml

#import sys     # pylint: disable=C0411
#import os.path # pylint: disable=C0411
#sys.path.append( os.path.join(os.path.dirname(__file__), '..') )

from cheap_pie.cheap_pie_core.cp_builder import CpHalBuilder # pylint: disable=C0413,E0401
from cheap_pie.cheap_pie_core.cp_cli import cp_devices_fname

def ipxact_remove_prefix(ipx):
    """ remove ipxact or spirit prefix"""
    replist = ['ipxact_','spirit_']

    if hasattr(ipx,'children'):
        for elm in ipx.children:
            # remove all suffixes
            for repl in replist:
                elm._name = elm._name.replace(repl,'') # pylint: disable=W0212
            # remove recursively
            ipxact_remove_prefix(elm)

    return ipx

def ipxact_parse(fname,hif=None, base_address_offset = "0x00000000"):
    """ Cheap Pie native module parser for IP-XACT """
    ## read input file ########################################################
    csv = untangle.parse(fname)

    ## remove ipxact/spirit prefixes ##########################################
    csv = ipxact_remove_prefix(csv)

    ## loop over lines ########################################################
    cpb = CpHalBuilder(hif)

    periph = csv.component.memoryMaps.memoryMap.addressBlock
    base_addr_str=periph.baseAddress.cdata.replace("'h",'0x')
    base_address=literal_eval(base_addr_str)

    if hasattr(periph,'register'): # pylint: disable=R1702
        for reg in periph.register:
            if hasattr( reg.name, 'cdata'):
                # new register
                addr_str=reg.addressOffset.cdata.replace("'h",'0x')
                regaddr=literal_eval(addr_str) + base_address + literal_eval(base_address_offset)

                cpb.reg_open(regname = f'{periph.name.cdata}_{reg.name.cdata}',
                             regaddr = regaddr)

                # for field_idx in range(nfields):
                if hasattr(reg,'field'):
                    for field in reg.field:
                        if not field is None:
                            # Create new field
                            cpb.newfield(
                                regfield = field.name.cdata,
                                width = field.bitWidth.cdata,
                                offset = field.bitOffset.cdata,
                                comments = "" # field.description.cdata
                                )

    # convert output dictionary into structure
    return cpb.out()

def test_ipxact_parse():
    """ Test function for cheap pie native IP-XACT parser """

    ipxact_parse(cp_devices_fname("my_subblock.xml"))
    ipxact_parse(cp_devices_fname("leon2_creg.xml"))
    ipxact_parse(cp_devices_fname("generic_example.xml"))

if __name__ == '__main__':
    import sys
    if len(sys.argv) > 1:
        LFNAME=sys.argv[1]
    else:
        LFNAME=cp_devices_fname("my_subblock.xml")
    print(LFNAME)
    print(ipxact_parse(LFNAME))
